
/*
 * Bird.java
 */

import info.gridworld.actor.Actor;
import info.gridworld.grid.Grid;
import info.gridworld.grid.Location;

/**
 * @author cday
 * @version June 4, 2007
 */
public class Bird extends Actor implements Flockable {
    
    private static final double TURNPCT = 0.75;
    private Actor myLeader = null;

    /**
     *  Instructs this Bird on how to behave when it is told to move
     */
    public void move() {
        Grid<Actor> gr = getGrid();
        if (gr == null)
            return;
        Location loc = getLocation();
        Location next = loc.getAdjacentLocation(getDirection());
        if (gr.isValid(next))
            moveTo(next);
        else
            removeSelfFromGrid();
    }
    
    /**
     *  Overrides the act method in parent class Actor.  This method is
     *  called on each "step" when a World is created
     **/
    public void act() {
        if (isFollowing()) {
            turnToFace(myLeader.getLocation());
        } else if (Math.random() > TURNPCT) {
            turn();
        }
        if (canMove()) {
            move();
        }
    }
    
    /**
     *  Overrides method turn in class Actor.  Turn 45 degrees to the left.
     **/
    public void turn() {
        setDirection(getDirection() + Location.HALF_LEFT);
    }
    
    /** Given a Location, turn to face that Location
     * @param   loc The location this Predator should face
     **/
    private void turnToFace(Location loc) {
        if (loc.equals(this.getLocation())) return;
        int xDist = this.getLocation().getCol() - loc.getCol();
        int yDist = this.getLocation().getRow() - loc.getRow();
        double turn = Math.atan2(yDist, xDist);
        setDirection((int)Math.toDegrees(turn) - 90);
    }
    
    /**
     *  Return true iff this Bird can move
     * 
     * 
     * @return true if this Bird can move, false if it cannot
     */
    public boolean canMove() {
        Grid<Actor> gr = getGrid();
        if (gr == null)
            return false;
        Location loc = getLocation();
        Location next = loc.getAdjacentLocation(getDirection());
        if (!gr.isValid(next))
            return false;
        Actor neighbor = gr.get(next);
        return (neighbor == null);
        
    }
    
    /**
     *  Instruct this Flockable object to permanently  stop following its leader
     **/
    public void stopFollowing() {
        myLeader = null;
    }
    
    /**
     *  Return true if this Flockable has a leader that it is following
     *  @return true if this Flockable has a leader, false otherwise
     **/
    public boolean isFollowing() {
        return myLeader != null;
    }
    
    /**
     *  Provide this Flockable object with a leader that it can follow.
     *  Subsequent actions by this Flockable object should cause it to behave
     *  in a manner that causes it to follow flockLeader
     *  @param  flockLeader The Actor that this Flockable will follow
     **/
    public void setLeader(Actor leader) {
        myLeader = leader;
    }
}
